////////////////////////////////////////////////////////////////////////////////
// Copyright 2019-2020 Lawrence Livermore National Security, LLC and other
// DiHydrogen Project Developers. See the top-level LICENSE file for details.
//
// SPDX-License-Identifier: Apache-2.0
////////////////////////////////////////////////////////////////////////////////

#pragma once
#ifndef H2_CONFIG_HPP_
#define H2_CONFIG_HPP_

// Version information
#define H2_VERSION_MAJOR 0
#define H2_VERSION_MINOR 3
#define H2_VERSION_PATCH 0

// Configuration information
#define H2_HAS_HALF 0
#define H2_HAS_OPENMP 0

#define H2_HAS_CUDA 0
#define H2_HAS_ROCM 0
#if H2_HAS_CUDA || H2_HAS_ROCM
#define H2_HAS_GPU
#endif

// Features detected at configure time
#define H2_PRETTY_FUNCTION __PRETTY_FUNCTION__
#define H2_RESTRICT __restrict__

// C++ standard
#ifdef __cplusplus

#if __cplusplus > 202100L
#define H2_CXX_STANDARD 23
#elif __cplusplus > 202000L
#define H2_CXX_STANDARD 20
#elif __cplusplus > 201700L
#define H2_CXX_STANDARD 17
#else
#error "C++17 or newer is required."
#endif

#if H2_CXX_STANDARD >= 20
#define H2_CONSTEXPR_FUNC constexpr
#if __has_cpp_attribute(nodiscard)
#define H2_NODISCARD [[nodiscard]]
#else
#define H2_NODISCARD
#endif
#else
#define H2_CONSTEXPR_FUNC
#define H2_NODISCARD
#endif

#endif // defined(__cplusplus)

#endif // H2_CONFIG_HPP_
